/****************************************************************************
**
** gfxlcdtest.c
**
** written 08/2000 by <cavofl@t-online.de>
**
** last changed: 14.01.2001
**
*****************************************************************************/

#include <curses.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/io.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <signal.h>
#include <math.h>

#include "gfxlcd.h"
#include "sed133x.h"
#include "font_pearl_8x8.c"

/*-----------------------------------------------------------------------------
 * name: dec2bin(int decimal_value, char* buffer)
 * purpose: convert decimal to binary
 * args:  decimal value to convert, buffer (at least 9 bytes!)
 * returns: (char*)buffer (string)
 *---------------------------------------------------------------------------*/
char* dec2bin(int dec, char *bin)
{
	int x;
	int bit = 128;
	
	for (x = 0; x < 8; x++)
		{
		 	if (dec & bit)
		 		bin[x] = '1';
		 	else 
				bin[x] = '0';
 
			bit = bit >> 1;
		 }
	bin[x] = '\0'; /* terminate the string */
	return bin;
}

/*-----------------------------------------------------------------------------
 * name: prbin(char* text, int dec)
 * purpose: textoutput and call to dec2bin
 * args: text to print, decimalvalue to convert
 * returns: nothing
 *---------------------------------------------------------------------------*/
void prbin(char *text, int dec)
{
	char* out = (char*)calloc(9, sizeof(char));
	out = dec2bin(dec, out);
	wprintw(logw, "%s:%d:%s\n", text, dec, out);
	wrefresh(logw);
	free(out); 
}

/*-----------------------------------------------------------------------------
 * name: lcd_write_cmd(u8 command_byte)
 * purpose: sends the command byte to the display
 * args: command to send
 * ---------------------------------------------------------------------------*/
void lcd_write_cmd(u8 command)
{
 if (debug) {
	 wprintw(logw, "lwc: writing command: 0x%x\n", command);
 	 wrefresh(logw);
 }
	 
 WRLO;		/* to avoid conflicts with the CFL/RESET-Logic */
 
 A0HI;		/* A0 to HIGH = command */
 outb(command, DATA);	/* write command byte to data port */
 CSLO;		/* CS to low */
 WRLO;		/* WR to low */
 WRHI;		/* WR to high */
 CSHI;		/* CS to high */
 A0LO;
}

/*-----------------------------------------------------------------------------
 * name: lcd_read_byte()
 * purpose: reads one byte from port
 * args: none
 * returns: readed byte
 * comments: IMPORTANT: Make sure RD is low before CS is going to low,
 * 		        because otherwise the RESET-Logic may be triggered,
 *			if bit 7 is set. THIS MUST NOT BE HAPPEN! IF RESET
 *			IS PULLED LOW, THE LCD GETS NO MORE CLOCK-PULSES AND
 *			MAY BE DAMAGED!
 *			After reading the port, make sure CS is toggled to HIGH
 *			before RD is taken HIGH, otherwise the same thing maybe
 *			happen!
 * ---------------------------------------------------------------------------*/
u8 lcd_read_byte()
{
 
 u8 data_byte = 0;
 
 if (debug)
	 logit("lrb: reading from port");
 
 DATAIN;	/* port to input */
 
 A0HI;		/* A0 to HIGH = Display and cursor address read */
 RDLO;		/* Make sure, RD is LOW before CS is lowered! */
 WRHI;
 CSLO;		/* CS to low */
 RDLO;		/* RD to low */
  
 data_byte = inb(DATA);
  
 CSHI;		/* CS to high */
 		/* Make sure, CS is HIGH before RD is going HIGH! */

 RDHI;		/* RD to high */
 DATAOUT;	/* port to output */

 return data_byte; 
}

/*-----------------------------------------------------------------------------
 * name: lcd_get_status()
 * purpose: reads status byte from port
 * args: none
 * returns: status byte (see SED133x manual)
 * ---------------------------------------------------------------------------*/
u8 lcd_get_status(void)
{
 u8 status_byte;
 
 DATAIN;
 
 A0LO;		/* A0 to LOW = status byte read */
 WRHI;		/* ! */
 CSLO;		/* CS to low */
 RDLO;		/* RD to low */
 
 status_byte = inb(DATA);
 
 RDHI;		/* RD to high */
 CSHI;		/* CS to high */
 DATAOUT;

 return status_byte; 
}

/*-----------------------------------------------------------------------------
 * name: lcd_write_command(u8 command_byte, int plen, u8 parameter[])
 * purpose: sends the command byte and it's parameter(s) to the display
 * args: command to send, number of parameters, parameter array
 * ---------------------------------------------------------------------------*/
void lcd_write_command(u8 command, int plen, u8 parameter[])
{
 int i;
 
 lcd_write_cmd(command);
 if (plen != 0) {
	for (i = 0; i < plen; i++) {
		lcd_write_data(parameter[i]);
	 	if (debug) {
			wprintw(logw, "lwco: parameter: 0x%x\n", parameter[i]);
			wrefresh(logw);
		}
	}
 }
}

/*-----------------------------------------------------------------------------
 * name: lcd_write_data(u8 data_byte)
 * purpose: sends the data byte to the display
 * args: byte to send
 * ---------------------------------------------------------------------------*/
void lcd_write_data(u8 databyte)
{
   /* wait for status flag */
   while ((lcd_get_status() & (u8)64) == 64) {}
   
   A0LO;		/* A0 to LOW = data */
   outb(databyte, DATA);	/* write data byte to data port */
   CSLO;		/* CS to low */
   WRLO;		/* WR to low */
   WRHI;		/* WR to high */
   CSHI;		/* CS to high */
}

/*-----------------------------------------------------------------------------
 * name: lcd_clear_text()
 * purpose: clears text display RAM area (1200 characters 40x30)
 * ---------------------------------------------------------------------------*/
void lcd_clear_text(void)
{
 int i;
 u8 para[10];
 
 if (debug)
	 logit("lct: clearing display...\n");
 
 para[0] = 0x00;
 para[1] = 0x00;
 lcd_write_command(CSRW, 2, para);	/* set cursor to text page */
 lcd_write_cmd(MWRITE);			/* mwrite */

 /* write 1200 spaces */
 for (i = 0; i < 1200; i++) {
 	lcd_write_data((u8)0x20);
 }
}

/*-----------------------------------------------------------------------------
 * name: lcd_set_gfx_csr(x,y)
 * purpose: set the pixel at the given coordinates
 * args: x and y
 * ---------------------------------------------------------------------------*/
void lcd_set_gfx_csr(int x,int y)
{
 int addr, addr_x;
 int gcsr, byte, old_byte;
 u8 para[2];
  
 /* calc csr address */
 addr = (x/8+(y*LCD_TEXT_COLS)+LCD_GFX_ADDRESS);
 addr_x = x%8;
 
 if (addr_x == 0)
 	gcsr = 1;
 else
 	gcsr = pow(2, addr_x);
	
 byte = 128/gcsr;
  
 /* place csr */
 para[0] = addr%256;
 para[1] = (addr >> 8);
 lcd_write_command(CSRW, 2, para);
 
 /* read old byte from csr address */
 lcd_write_cmd(MREAD);
 old_byte = lcd_read_byte();
 
 /* place csr */
 lcd_write_command(CSRW, 2, para);
 
 /* or bytes */
 byte |= old_byte;
 
 lcd_write_cmd(MWRITE);
 lcd_write_data(byte);
}

/*-----------------------------------------------------------------------------
 * name: lcd_write_image(int size)
 * purpose: write a 320x240 (9600 bytes) sized image into graphic RAM
 * args: size of image
 * comment: fmap must be a valid mem-pointer
 * ---------------------------------------------------------------------------*/
void lcd_write_image(int size)
{
 int i;
 u8 para[2];
 
 if (fmap == NULL) {
	 logit("liw: error: fmap is NULL");
	 finish(0);
 }
 
 if (size > 9600)
	 logit("lwi: warning: image is to big. will be cutted!\n");
 if (debug)
	 logit("Writing image to display...\n");
 
 para[0] = 0xb0;	/* set cursor to graphic address */
 para[1] = 0x04;
 lcd_write_command(CSRW, 2, para);

 lcd_write_cmd(MWRITE);		/* mwrite */
 for (i = 0; i < 9600; i++) {
	lcd_write_data(fmap[i]);
 }
 
}

/*-----------------------------------------------------------------------------
 * name: lcd_write_text(int tx, ty, char *text)
 * purpose: write a text string to display text area
 * args: x pos, y pos, text
 * ---------------------------------------------------------------------------*/
void lcd_write_text(int tx, int ty, char *text)
{
 u8 par[2];
 int tlen = 0, lsb, msb, i;
 int addrstart = 0, addrend = 0;
 int tareasize = LCD_TEXT_ROWS*LCD_TEXT_COLS;
 
 tlen = strlen(text);
 if (debug) {
	 wprintw(logw, "lwt: x: %d, y: %d, text: %s, len: %d\n", tx, ty, text, tlen);
	 wrefresh(logw);
 }

 addrstart = tx + (ty * LCD_TEXT_COLS);
 addrend   = addrstart + tlen;
 
 if (debug) {
	 wprintw(logw, "addrstart: %d, addrend: %d\n", addrstart, addrend);
	 wrefresh(logw);
 }

  if (addrend > tareasize) {
	  logit("warning: addrend to long, text will be cutted.\n");
	  addrend = tareasize;
  }

  /* place cursor */
  lsb = (u8)addrstart;
  msb = (addrstart >> 8); 
  wprintw(logw, "addrstart: 0x%x lsb: 0x%x msb: 0x%x\n", addrstart, lsb, msb);
  wrefresh(logw);
  
   par[0] = (u8)lsb;
   par[1] = (u8)msb;
   lcd_write_command(CSRW, 2, par);

  /* write text */
  lcd_write_command(MWRITE, 0, NULL);
  for (i = 0; i < tlen; i++) {
	  lcd_write_data(text[i]);
  }
}

/*-----------------------------------------------------------------------------
 * name: loadimage(char *file)
 * purpose: loads file and maps it into memory
 * args: file to load
 * returns: size of file/mapped memory
 * comment: fmap (pointer to mapped ram) is a static variable
 *          image has to be a pbm file _without_ header!
 *---------------------------------------------------------------------------*/
int loadimage(char *file)
{
 int fd;
 int filesize;
 struct stat statbuffer;

 if ((fd = open(file, O_RDONLY)) == -1) {
	 logit("li: Can't open file");
	 finish(0);
 }

 if (fstat(fd, &statbuffer) == -1) {
	 logit("li: Can't stat file");
	 close(fd);
	 finish(0);
 }
 
 filesize = statbuffer.st_size;
 if (debug) {
	 wprintw(logw, "li: filesize: %d\n", filesize);
	 wrefresh(logw);
 }

 fmap = mmap(0, filesize, PROT_READ, MAP_SHARED, fd, 0);
 if (fmap == (u8*)-1) {
	 logit("li: error during mmap");
	 close(fd);
	 finish(0);
 }
 if (debug) {
	wprintw(logw, "li: mapped memory at: 0x%x\n", (int)fmap);
 	wrefresh(logw);
 }

 close(fd);
 return filesize;
}

/*-----------------------------------------------------------------------------
 * name: lcd_init()
 *---------------------------------------------------------------------------*/
void lcd_init(void)
{
 u8 para[10];
 int i;
 
 ext_reset();	/* reset SED */
 
 DATAOUT;	/* Port to output */
 
 /* set output pins to an initial state */
 A0LO;	/* A0 is high active      */
 RDHI;	/* /RD line is low active */
 WRHI;	/* /WR line is low active */
 CSHI;	/* /CS line is low active */

 /*** init of display ***/
 
 /* SYSTEM_SET (cmd1, msg1) */
 para[0] = 0x31;	/* M0, M1, M2, W/S, 1, IV, T/L, DR */
 para[1] = 0x87;	/* FX, FX, FX, FX,  0,  0,   0, WF */
 para[2] = 0x07;	/* FY, FY, FY, FY,  0,  0,   0,  0 */
 para[3] = 0x27;	/* C/R */
 para[4] = 0x39;	/* TC/R */
 para[5] = 0xef;	/* L/F */
 para[6] = 0x28;	/* APL */
 para[7] = 0x00;	/* APH */
 lcd_write_command(SYSTEM_SET, 8, para);
 
 /* SCROLL (cmd2, msg2) */
 para[0] = 0x00;	/* SAD1L */	
 para[1] = 0x00;	/* SAD1H */
 para[2] = 0xf0;	/* SL1 */
 para[3] = 0xb0;	/* SAD2L */
 para[4] = 0x04;	/* SAD2H */
 para[5] = 0xf0;	/* SL2 */
 para[6] = 0x00;	/* SAD3L */
 para[7] = 0x00;	/* SAD3H */
 para[8] = 0x00;	/* SAD4L */
 para[9] = 0x00;	/* SAD4H */
 lcd_write_command(SCROLL, 10, para);

 /* CSRFORM (cmd3, msg3) */
 para[0] = 0x04;	/* CRX, 0, 0, 0, 0  */
 para[1] = 0x86;	/* CRY, 0, 0, 0, CM */
 lcd_write_command(CSRFORM, 2, para);
 
 /* CGRAM ADDRESS = 0xf000 */
 para[0] = 0x00;
 para[1] = 0xf0;
 lcd_write_command(CGRAM_ADR, 2, para);
 
 /* CSRDIR_RIGHT (cmd4) */
 lcd_write_command(CSRDIR_RIGHT, 0, NULL);

 /* HDOT_SCR (cmd5, msg18) */
 para[0] = 0x00;	/* D0, D1, D2, D3, 0, 0, 0, 0 */
 lcd_write_command(HDOT_SCR, 1, para);

 /* OVLAY (cmd6, msg4) 0x01 */
 para[0] = 0x01;	/* MX0, MX1, DM1, DM2, OV, 0, 0, 0 */
 lcd_write_command(OVLAY, 1, para);

 /* load font */
 para[0] = 0x00;
 para[1] = 0xf0;
 lcd_write_command(CSRW, 2, para);
 
 lcd_write_cmd(MWRITE);
 for (i = 0; i<FONTDATAMAX; ++i) {
 	lcd_write_data(fontdata_pearl8x8[i]);
 }
 wprintw(logw, "%d bytes of font data written\n", i);
 wrefresh(logw);
 
 /* DISP_ON (cmd7, msg5) 0x16 */
 para[0] = 0x16;	/* FC0, FC1, FP0, FP1, FP2, FP3, FP4, FP5 */
 lcd_write_command(DISP_ON, 1, para);
 
}

/*-----------------------------------------------------------------------------
 * name: lcd_console()
 *---------------------------------------------------------------------------*/
void lcd_console(void)
{
  u8 para[10];
  u8 csr_l, csr_h;
  int i;
  int end = 1;
  char c = ' ';
  int csraddr = 0;
  int x = 0, y = -1;
  int s_size = LCD_TEXT_ROWS*LCD_TEXT_COLS;
  char screen[s_size];
  int scraddr = 0;
  
  logit("Entering console mode. Enter . for end.\n");
  para[0] = 0x00;
  para[1] = 0x00;
  lcd_write_command(CSRW, 2, para);
  lcd_write_cmd(MWRITE);
  
  wprintw(logw, "Sreensize: %d\n", s_size);
  wrefresh(logw);
  
  for (i = 0; i < s_size; i++) {
  	screen[i] = 0x20;
  }
  fflush(stdin);
  while (end) {
	c = getch();
	if (c == '.')
		end = 0;
	if (c == '\r') {
		y++;
		if (y > 29) {
			/* DISP OFF */
			para[0] = 0x00;
	     		lcd_write_command(DISP_OFF, 1, para);
			
			for (i=0; i<s_size-40; i++) {
				screen[i] = screen[i+40];
			}
			
			for (i=0; i<40; i++) {
				screen[i+1159] = 0x20;
			}
			para[0] = 0x00;
			para[1] = 0x00;
			lcd_write_command(CSRW, 2, para);
			lcd_write_cmd(MWRITE);
			for (i=0; i<s_size; i++) {
				lcd_write_data(screen[i]);
			}
			
			/* DISP ON */
			para[0] = 0x16;
	     		lcd_write_command(DISP_ON, 1, para);
			y--;
		}
		
		x = 0;
		csraddr = x + (y * LCD_TEXT_COLS);
		csr_l = (u8)csraddr;
  		csr_h = (csraddr >> 8); 
   		para[0] = csr_l;
   		para[1] = csr_h;
   		lcd_write_command(CSRW, 2, para);
		lcd_write_cmd(MWRITE);
		scraddr = x + (y * LCD_TEXT_COLS);
		/* screen[scraddr] = 0x20; */
		wprintw(logw, "screenadr: 0x%x\n", scraddr);
		wrefresh(logw);
	} else {
		lcd_write_data(c);
		scraddr = x + (y * LCD_TEXT_COLS);
		screen[scraddr] = c;
		wprintw(logw, "screenadr: 0x%x\n", scraddr);
		wrefresh(logw);
		x++;
		if (x >= 40) {
			x = 0;
			y++;
		}
			
	}
	wprintw(logw, "x: %d, y %d\n", x, y);
	wrefresh(logw);
  }
  logit("Console Mode ends.");
}

/*-----------------------------------------------------------------------------
 * name: gfx_test()
 * purpose: draw a cool sinus curve
 *---------------------------------------------------------------------------*/
void gfx_test(void)
{
 int i, x ,y;
 for (i=10; i<310 ;++i)
 	lcd_set_gfx_csr(i,120);
 
 for (i=10; i<230; i++)
 	lcd_set_gfx_csr(10, i);

 i = 1;
 for(x=10; x<310; ++x) {
 	y=120+10*(10*sin(2*3.1415*i*120));
	lcd_set_gfx_csr(x, y);
	i++;
 }
}

/*-----------------------------------------------------------------------------
 * name: lcd_convert_attr(u16 ch)
 * purpose: for attribute testing - stolen from mdacon.c
 *---------------------------------------------------------------------------*/
u16 lcd_convert_attr(u16 ch)
{
    /* only reverse, blinking and reverse+blinking supported */
    /* underline simulated by reverse */
    u16 attr=0;

    if (ch & 0x8000) {
        // blink
        attr |= 0x0800;
    }
    if (ch & 0x1000) {
        // reverse
        attr |= 0xff00;
    }
    if (ch & 0x0800) {
        // underline, simulate by reverse
        attr |= 0x0500;
    }
    wprintw(logw, "lca: ch: 0x%x attr: 0x%x\n", ch, attr);
    wrefresh(logw);
    return (ch & 0x00ff) | attr;
}

/*-----------------------------------------------------------------------------
 * name: lcd_attr_test(u16 ch, int x, int y, int a)
 * purpose: for attribute testing (tests for mdacon.c)
 *---------------------------------------------------------------------------*/
void lcd_attr_test(u16 ch, int x, int y, int a)
{
  u8 para[10];
  u16 ch_converted;
  u16 position;
  u16 a_position;
  u16 i;
  
  position=POSITION(x,y);
  a_position=A_POSITION(x,y);
  ch_converted=lcd_convert_attr(ch); 

    /* write character attribute to LCD */
        
    /* set cursor */
    para[0] = (LCD_GFX_ADDRESS+a_position)%256;
    para[1] = (LCD_GFX_ADDRESS+a_position)>>8;
    lcd_write_command(CSRW, 2, para);
    
    /* set direction down */
    lcd_write_cmd(CSRDIR_DOWN);
    
    lcd_write_cmd(MWRITE);
    for (i=0; i<8; ++i) {	
    	lcd_write_data(ch_converted>>8);
    }
    
    lcd_write_cmd(CSRDIR_RIGHT);
    wprintw(logw, "putc: attribute: 0x%x\n", ch_converted>>8);
    wrefresh(logw);

    /* write character code to LCD */
    para[0] = (LCD_TEXT_ADDRESS+position)%256;
    para[1] = (LCD_TEXT_ADDRESS+position)>>8;
    lcd_write_command(CSRW, 2, para);
    lcd_write_cmd(MWRITE);
    lcd_write_data(ch_converted & 0x00ff);

    wprintw(logw, "putc: character: 0x%x\n", ch_converted & 0x00ff);
    wrefresh(logw);
}

/*-----------------------------------------------------------------------------
 * name: ext_ctrl(u8 ext_command)
 * purpose: CFL control (with extra control circuit only!)
 *---------------------------------------------------------------------------*/
void ext_ctrl(u8 cmd)
{
	DATAOUT;
	WRHI;
	RDHI;
	A0HI;
	outb(cmd | 0x80, DATA);
	CSLO;
	CSHI;
	A0LO;
}

/*-----------------------------------------------------------------------------
 * name: ext_reset()
 * purpose: RESET control (with extra control circuit only!)
 *---------------------------------------------------------------------------*/
void ext_reset(void)
{
	DATAOUT;
	WRHI;
	RDHI;
	A0HI;
	outb(0x81, DATA); /* reset SED */
	CSLO;
	CSHI;
	usleep(10);
	outb(0x83, DATA);
	CSLO;
	CSHI;
	A0LO;
}

/*-----------------------------------------------------------------------------
 * name: main
 *---------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
 int fs, i, y;
 int c;
 u8 pbyte[3];
 u8 para[10];
 
 /* ncurses init */
 (void) signal(SIGINT, finish);
 mainw = initscr();
 keypad(stdscr, TRUE);
 (void) nonl();
 (void) cbreak();
 (void) noecho();
 
 if (has_colors()) {
	 start_color();
	 init_pair(COLOR_BLACK, COLOR_BLACK, COLOR_BLACK);
	 init_pair(COLOR_GREEN, COLOR_GREEN, COLOR_BLACK);
	 init_pair(COLOR_RED, COLOR_YELLOW, COLOR_BLUE);    /* 1 */
	 init_pair(COLOR_CYAN, COLOR_CYAN, COLOR_BLACK);
	 init_pair(COLOR_WHITE, COLOR_WHITE, COLOR_BLACK);
	 init_pair(COLOR_MAGENTA, COLOR_MAGENTA, COLOR_BLACK);
	 init_pair(COLOR_BLUE, COLOR_BLUE, COLOR_BLACK);
	 init_pair(COLOR_YELLOW, COLOR_YELLOW, COLOR_BLACK);
 }
 
 wrefresh(mainw);
 refresh();

 keyw = newwin(13, COLS, 0, 0);
 logw = newwin(LINES-13, COLS, 13, 0);
 wattron(keyw,COLOR_PAIR(1)|A_BOLD);
 
 /* there must exists an easier way to do this :-/ */
 for (y = 0; y < 13; y++) {
     for (i = 0; i < COLS; i++) {
	 waddstr(keyw, " ");
     }
 }
 box(keyw, ACS_VLINE, ACS_HLINE);
  
 mvwprintw(keyw, 1, 2, "1 - Display on             2 - Display off");
 mvwprintw(keyw, 2, 2, "3 - Enter Sleep Mode       4 - Wakeup from Sleep Mode");
 mvwprintw(keyw, 3, 2, "5 - Display default GFX    6 - Display default Text");
 mvwprintw(keyw, 4, 2, "7 - Clear GFX Screen       8 - Clear Text Screen");
 mvwprintw(keyw, 5, 2, "9 - Cursor off             0 - Cursor on");
 mvwprintw(keyw, 6, 2, "c - Console Mode           r - read Cursor address");
 mvwprintw(keyw, 7, 2, "g - GFX Test               s - read Status flag");
 mvwprintw(keyw, 8, 2, "o - read control port      b - get byte at cursor address");
 mvwprintw(keyw, 9, 2, "a - attribute test         e - Reset SED");
 mvwprintw(keyw, 10, 2, "w - Backlight on           v - Backlight off");
 mvwprintw(keyw, 11, 2, "x - Exit");

 mvwprintw(keyw, 12, COLS-20, " gfxlcdtest 0.7 ");
 
 scrollok(logw, TRUE);
 wrefresh(keyw);
 wrefresh(logw);

 logit("ncurses init done");
  
 /* init */

 /* get io permissions */
 if (ioperm(BASE, 3, 1)) {
	 logit("Can't get io permissions");
	 finish(0);
 }
 
 fs = loadimage("testbild2_bw.pbm");
 logit("Image loading done");
 
 lcd_init();
 logit("lcd_init() done");
 
 for (;;)
 {
   c = getch();  

   switch(c) {
     case '1' :
	     logit("Turning Display on");
	     para[0] = 0x16;
	     lcd_write_command(DISP_ON, 1, para);
	     break;
     case '2' :
	     logit("Turning Display off");
	     para[0] = 0x00;
	     lcd_write_command(DISP_OFF, 1, para);
	     break;
     case '5' :
	     logit("Displaying graphics");
	     /*** display bitmap graphic ***/
 	     /* load image */
 	     lcd_write_image(fs);
	     break;
     case '6' :
	     logit("Displaying default text");
	     /*** display text ***/
 	     lcd_write_text(0,22, "LCD MODULE");
 	     lcd_write_text(0,23, "Powered and driven");
 	     lcd_write_text(0,24, "by the Penguin.");
      	     break;
     case '3' :
	     logit("Entering Sleep Mode");
	     lcd_write_command(SLEEP_IN, 0, NULL);
	     break;
     case '4' :
	     logit("Wakeup from Sleep Mode");
	     para[0] = 0x31;
	     lcd_write_command(SYSTEM_SET, 1, para);
	     para[0] = 0x16;
	     lcd_write_command(DISP_ON, 1, para);
	     break;
     case '8' :
	     logit("Clearing Text Screen");
	     lcd_clear_text();
	     break;
     case '7' :
	     logit("Clearing GFX Screen");
	     para[0] = 0xb0;
	     para[1] = 0x04;
	     lcd_write_command(CSRW, 2, para);
	     lcd_write_cmd(MWRITE);
	     for (i = 0; i < 9600; i++)
		     lcd_write_data(0);
	     break;
     case '9' :
     	     logit("Cursor off");
	     para[0] = lcd_display_mode &= ~CURSOR_ON & ~BLINK_ON;
	     lcd_write_command(DISP_ON, 1, para);
	     break;
     case '0' :
     	     logit("Cursor on");
	     para[0] = lcd_display_mode |= BLINK_ON & ~CURSOR_ON;
	     lcd_write_command(DISP_ON, 1, para);
	     break;     
     case 'c' :
	     lcd_console();
	     break;
     case 'r' :
	     logit("reading from port");
	     lcd_write_cmd(CSRR);
	     pbyte[0] = lcd_read_byte();
	     pbyte[1] = lcd_read_byte();
	     wprintw(logw, "Bytes read: L: 0x%x H: 0x%x\n", pbyte[0], pbyte[1]);
	     wrefresh(logw);
	     break;
     case 'b' :
     	     logit("reading byte from cursor address");
	     lcd_write_cmd(MREAD);
	     pbyte[0] = lcd_read_byte();
	     wprintw(logw, "Byte under cursor: 0x%x (%c)\n", pbyte[0], pbyte[0]);
	     wrefresh(logw);
	     break;
     case 's' :
     	     logit("reading status flag");
	     pbyte[0] = lcd_get_status();
	     prbin("flag", pbyte[0]);
	     break;
     case 'g' :
     	     logit("gfx testing");
	     gfx_test();
	     break;
     case 'o' :
	     logit("reading control port");
     	     pbyte[0] = inb(CONTROL);
	     prbin("ControlReg", pbyte[0]);
	     break;
     case 'a' :
	     logit("Attribute Test");
     	     lcd_attr_test(0x141 | 0x1000, 1, 0, 0);
	     lcd_attr_test(0x142 | 0x1000, 2, 0, 0);
	     lcd_attr_test(0x043 | 0x1000, 2, 1, 0);
	     break;
     case 'e' : /* reset SED */
     	     logit("Resetting SED");
	     ext_reset();
	     lcd_init(); /* after reset init is necessary */
     	     break;
     case 'w' : /* CFL on */
	     logit("Turning CFL on");
             ext_ctrl(CTRL_CFL_ON);
             break;
     case 'v' : /* CFL off */
	     logit("Turning CFL off");
             ext_ctrl(CTRL_CFL_OFF);
             break;
     case 'x' :
	     logit("Bye.");
	     if (fmap)
		 munmap(fmap, fs);
	     finish(0);
	     break;
     default : 
	     break;
   }
  }	     
 

 finish(0);
 return 0;
}

void finish(int sig)
{
    sleep(2);
    delwin(keyw);
    delwin(logw);
    delwin(mainw);
    endwin();
    exit(0);
}

void logit(char *text)
{
    wprintw(logw, "%s\n", text);
    wrefresh(logw);
}
